@@ -2,12 +2,13 @@ def hash_everything(l):
     try:
         return hash(l)
     except TypeError:
+        h = hash(l.__class__)
+        
         try:
             xs = iter(l)
         except TypeError:
-            return 0
+            return h
 
-        h = 0
         for x in xs:
             h = h ^ hash_everything(x)
         return h
@@ -20,6 +21,7 @@ class HashItAnyway(object):
     def __eq__(self, other): 
         return (isinstance(other,HashItAnyway) and
                 self.wrapped.__class__ == other.wrapped.__class__ and
+                self.h == other.h and
                 self.wrapped == other.wrapped)
 
     def __ne__(self,other):
